#ifndef KADDRESSBOOKMAIN_H
#define KADDRESSBOOKMAIN_H

#include <qptrlist.h>
#include <kapp.h>
#include <kmainwindow.h>
#include <kaction.h>

#include "kaddressbookiface.h"
#include "kaddressbook.h"

class ViewManager;

/**
 * This class serves as the main window for KAddressBook.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author Don Sanders <dsanders@kde.org>
 * @version 0.1
 */
class KAddressBookMain : public KMainWindow, virtual public KAddressBookIface
{
    Q_OBJECT
  public:
    KAddressBookMain();
    virtual ~KAddressBookMain();

  public slots:
    virtual void addEmail( QString addr ) { mWidget->addEmail( addr ); }

    virtual ASYNC showContactEditor( QString uid ) { mWidget->showContactEditor( uid ); }
    virtual void newContact() { mWidget->newContact(); }
    virtual QStringList getKeys() const { return mWidget->getKeys(); }
    virtual QDict<ContactEntry> getEntryDict() const { return mWidget->getEntryDict(); }
    virtual void addEntry( ContactEntry newEntry) { mWidget->addEntry( newEntry ); }
    virtual void changeEntry( QString key, ContactEntry changeEntry) { mWidget->changeEntry( key, changeEntry ); }
    virtual void removeEntry( QString key ) { mWidget->removeEntry( key ); }
    virtual QString getNameByPhone( QString phone ) { return mWidget->getNameByPhone( phone ); }
    virtual void save() { mWidget->save(); }
    virtual void exit() { close(); }
    virtual void updateEditMenu();

  protected:
    /**
     * This function is called when it is time for the app to save its
     * properties for session management purposes.
     */
    void saveProperties(KConfig *);

    /**
     * This function is called when this app is restored.  The KConfig
     * object points to the session management config file that was saved
     * with @ref saveProperties
     */
    void readProperties(KConfig *);

  protected slots:
    /** Called whenever an addressee is selected or unselected.
    *
    * @param selected True if there is an addressee select, false otherwise
    */
    void addresseeSelected(bool selected);
    
    /** Called whenever the addressbook is modified.
    *
    * @see KAddressBook
    */
    void modified(bool mod);
    
    /** Called whenever the view selection changes.
    */
    void selectViewAction();
    
    /** Called whenever the view configuration changes. This usually means
    * a view was added or deleted.
    */
    void viewConfigChanged(const QString &newActive);

    /** Called whenever the user clicks changes the view policy
    * of a quick tool.
    */
    void quickToolsAction();
    
    void configureToolbars();
    void configureKeys();

    void slotNewToolbarConfig();
        
  private slots:
    void clipboardDataChanged();

  private:
    void initActions();
    void initActionViewList();

    QString mActiveViewName;
    
    KAddressBook *mWidget;
    ViewManager *mViewManager;
    
    KAction *mActionPaste;
    KAction *mActionCut;
    KAction *mActionDelete;
    KAction *mActionCopy;
    KAction *mActionEditAddressee;
    KAction *mActionMail;
    KAction *mActionUndo;
    KAction *mActionRedo;
    KAction *mActionSave;
    KAction *mActionDeleteView;
    QPtrList<KAction> mActionViewList;
    KToggleAction *mActiveActionView;
    KToggleAction *mActionJumpBar;
    KToggleAction *mActionIncSearch;
};

#endif
